@@ -11,6 +11,7 @@ import (
 	"github.com/threefoldtech/zos/pkg/stubs"
 )
 
+// func headerRenderer(c zbus.Client, h *widgets.Paragraph, r *Flag) error {
 func headerRenderer(c zbus.Client, h *widgets.Paragraph, r *Flag) error {
 	env, err := environment.Get()
 	if err != nil {
@@ -22,14 +23,18 @@ func headerRenderer(c zbus.Client, h *widgets.Paragraph, r *Flag) error {
 	var farm string
 	farmID, err := identity.FarmID()
 	if err != nil {
-		farm = "not attached to a farm"
+		farm = "not set"
 	} else {
 		farm = fmt.Sprintf("%d", farmID)
 	}
 
-	format := fmt.Sprintf("Zero OS [%s] Version: %%s NodeID: %s FarmID: %s", env.RunningMode.String(), nodeID.Identity(), farm)
+	h.Text = "\n    Fetching realtime node information... please wait."
 
-	h.Text = "Zero OS"
+	var s string
+	s = "          Welcome to [Zero-OS](fg:yellow), [ThreeFold](fg:blue) Autonomous Operating System\n" +
+		"\n" +
+		" This is node [%s](fg:green) (farmer [%s](fg:green))\n" +
+		" running Zero-OS version [%s](fg:blue) (mode [%s](fg:cyan))"
 
 	host := stubs.NewVersionMonitorStub(c)
 	ctx := context.Background()
@@ -40,11 +45,8 @@ func headerRenderer(c zbus.Client, h *widgets.Paragraph, r *Flag) error {
 
 	go func() {
 		for version := range ch {
-			v := fmt.Sprintf(format, version.String())
-			if h.Text != v {
-				h.Text = v
-				r.Signal()
-			}
+			h.Text = fmt.Sprintf(s, nodeID, farm, version.String(), env.RunningMode.String())
+			r.Signal()
 		}
 	}()
 
