@@ -164,13 +164,19 @@ func (n *networker) Ready() error {
 	return nil
 }
 
+func (n *networker) ipv4Only() bool {
+	_, ipv4Only := n.ndmz.(*ndmz.Hidden)
+	return ipv4Only
+}
+
 func (n *networker) Join(networkdID pkg.NetID, containerID string, cfg pkg.ContainerNetworkConfig) (join pkg.Member, err error) {
 	// TODO:
 	// 1- Make sure this network id is actually deployed
-	// 2- Create a new namespace, then create a veth pair inside this namespace
-	// 3- Hook one end to the NR bridge
-	// 4- Assign IP to the veth endpoint inside the namespace.
-	// 5- return the namespace name
+	// 2- Check if the requested network config is doable
+	// 3- Create a new namespace, then create a veth pair inside this namespace
+	// 4- Hook one end to the NR bridge
+	// 5- Assign IP to the veth endpoint inside the namespace.
+	// 6- return the namespace name
 
 	log.Info().Str("network-id", string(networkdID)).Msg("joining network")
 
@@ -179,6 +185,10 @@ func (n *networker) Join(networkdID pkg.NetID, containerID string, cfg pkg.Conta
 		return join, errors.Wrapf(err, "couldn't load network with id (%s)", networkdID)
 	}
 
+	if cfg.PublicIP6 && n.ipv4Only() {
+		return join, errors.Errorf("this node runs in IPv4 only mode and you asked for a public IPv6. Impossible to fulfill the request")
+	}
+
 	netRes, err := nr.New(localNR)
 	if err != nil {
 		return join, errors.Wrap(err, "failed to load network resource")
@@ -189,7 +199,12 @@ func (n *networker) Join(networkdID pkg.NetID, containerID string, cfg pkg.Conta
 		ips[i] = net.ParseIP(addr)
 	}
 
-	join, err = netRes.Join(containerID, ips, cfg.PublicIP6)
+	join, err = netRes.Join(nr.ContainerConfig{
+		ContainerID: containerID,
+		IPs:         ips,
+		PublicIP6:   cfg.PublicIP6,
+		IPv4Only:    n.ipv4Only(),
+	})
 	if err != nil {
 		return join, errors.Wrap(err, "failed to load network resource")
 	}
@@ -417,7 +432,102 @@ func (n *networker) RemoveTap(networkID pkg.NetID) error {
 	return ifaceutil.Delete(tapIface, nil)
 }
 
-// GetSubnet of a local network resource identified by the network ID
+func (n *networker) PublicIPv4Support() bool {
+	return n.ndmz.SupportsPubIPv4()
+}
+
+// SetupPubTap sets up a tap device in the host namespace for the public ip
+// reservation id. It is hooked to the public bridge. The name of the tap
+// interface is returned
+func (n *networker) SetupPubTap(pubIPReservationID string) (string, error) {
+	log.Info().Str("pubip-res-id", string(pubIPReservationID)).Msg("Setting up public tap interface")
+
+	if !n.ndmz.SupportsPubIPv4() {
+		return "", errors.New("can't create public tap on this node")
+	}
+
+	bridgeName := n.ndmz.IP6PublicIface()
+
+	tapIface, err := pubTapName(pubIPReservationID)
+	if err != nil {
+		return "", errors.Wrap(err, "could not get network namespace tap device name")
+	}
+
+	_, err = tuntap.CreateTap(tapIface, bridgeName)
+
+	return tapIface, err
+}
+
+// PubTapExists checks if the tap device for the public network exists already
+func (n *networker) PubTapExists(pubIPReservationID string) (bool, error) {
+	log.Info().Str("pubip-res-id", string(pubIPReservationID)).Msg("Checking if public tap interface exists")
+
+	tapIface, err := pubTapName(pubIPReservationID)
+	if err != nil {
+		return false, errors.Wrap(err, "could not get network namespace tap device name")
+	}
+
+	return ifaceutil.Exists(tapIface, nil), nil
+}
+
+// RemovePubTap removes the public tap device from the host namespace
+// of the networkID
+func (n *networker) RemovePubTap(pubIPReservationID string) error {
+	log.Info().Str("pubip-res-id", string(pubIPReservationID)).Msg("Removing public tap interface")
+
+	tapIface, err := pubTapName(pubIPReservationID)
+	if err != nil {
+		return errors.Wrap(err, "could not get network namespace tap device name")
+	}
+
+	return ifaceutil.Delete(tapIface, nil)
+}
+
+// DisconnectPubTap disconnects the public tap from the network. The interface
+// itself is not removed and will need to be cleaned up later
+func (n *networker) DisconnectPubTap(pubIPReservationID string) error {
+	log.Info().Str("pubip-res-id", string(pubIPReservationID)).Msg("Disconnecting public tap interface")
+
+	tapIfaceName, err := pubTapName(pubIPReservationID)
+	if err != nil {
+		return errors.Wrap(err, "could not get network namespace tap device name")
+	}
+
+	tap, err := netlink.LinkByName(tapIfaceName)
+	if _, ok := err.(netlink.LinkNotFoundError); ok {
+		return nil
+	} else if err != nil {
+		if errors.Is(err, os.ErrNotExist) {
+			return nil
+		}
+		return errors.Wrap(err, "could not load tap device")
+	}
+
+	return netlink.LinkSetNoMaster(tap)
+}
+
+// GetPublicIPv6Subnet returns the IPv6 prefix op the public subnet of the host
+func (n *networker) GetPublicIPv6Subnet() (net.IPNet, error) {
+	// TODO
+	pubIface, err := netlink.LinkByName(n.ndmz.IP6PublicIface())
+	if err != nil {
+		return net.IPNet{}, errors.Wrap(err, "could not load public interface")
+	}
+	// get the ipv6 address
+	addrs, err := netlink.AddrList(pubIface, netlink.FAMILY_V6)
+	if err != nil {
+		return net.IPNet{}, errors.Wrap(err, "could not load public interface ipv6 addresses")
+	}
+	for _, addr := range addrs {
+		if addr.IP.IsGlobalUnicast() && !isULA(addr.IP) {
+			return *addr.IPNet, nil
+		}
+	}
+	return net.IPNet{}, nil
+}
+
+// GetSubnet of a local network resource identified by the network ID, ipv4 and ipv6
+// subnet respectively
 func (n networker) GetSubnet(networkID pkg.NetID) (net.IPNet, error) {
 	localNR, err := n.networkOf(string(networkID))
 	if err != nil {
@@ -427,25 +537,45 @@ func (n networker) GetSubnet(networkID pkg.NetID) (net.IPNet, error) {
 	return localNR.Subnet.IPNet, nil
 }
 
-// GetDefaultGwIP returns the IP(v4) of the default gateway inside the network
-// resource identified by the network ID on the local node
-func (n networker) GetDefaultGwIP(networkID pkg.NetID) (net.IP, error) {
+// GetNet of a network identified by the network ID
+func (n networker) GetNet(networkID pkg.NetID) (net.IPNet, error) {
 	localNR, err := n.networkOf(string(networkID))
 	if err != nil {
-		return nil, errors.Wrapf(err, "couldn't load network with id (%s)", networkID)
+		return net.IPNet{}, errors.Wrapf(err, "couldn't load network with id (%s)", networkID)
+	}
+
+	return localNR.NetworkIPRange.IPNet, nil
+}
+
+// GetDefaultGwIP returns the IPs of the default gateways inside the network
+// resource identified by the network ID on the local node, for IPv4 and IPv6
+// respectively
+func (n networker) GetDefaultGwIP(networkID pkg.NetID) (net.IP, net.IP, error) {
+	localNR, err := n.networkOf(string(networkID))
+	if err != nil {
+		return nil, nil, errors.Wrapf(err, "couldn't load network with id (%s)", networkID)
 	}
 
 	// only IP4 atm
 	ip := localNR.Subnet.IP.To4()
 	if ip == nil {
-		return nil, errors.New("nr subnet is not valid IPv4")
+		return nil, nil, errors.New("nr subnet is not valid IPv4")
 	}
 
 	// defaut gw is currently implied to be at `x.x.x.1`
 	// also a subnet in a NR is assumed to be a /24
 	ip[len(ip)-1] = 1
 
-	return ip, nil
+	// ipv6 is derived from the ipv4
+	return ip, nr.Convert4to6(string(networkID), ip), nil
+}
+
+// GetIPv6From4 generates an IPv6 address from a given IPv4 address in a NR
+func (n networker) GetIPv6From4(networkID pkg.NetID, ip net.IP) (net.IPNet, error) {
+	if ip.To4() == nil {
+		return net.IPNet{}, errors.New("invalid IPv4 address")
+	}
+	return net.IPNet{IP: nr.Convert4to6(string(networkID), ip), Mask: net.CIDRMask(64, 128)}, nil
 }
 
 // Addrs return the IP addresses of interface
@@ -961,3 +1091,20 @@ func tapName(netID pkg.NetID) (string, error) {
 	}
 	return name, nil
 }
+
+func pubTapName(resID string) (string, error) {
+	name := fmt.Sprintf("p-%s", resID)
+	if len(name) > 15 {
+		return "", errors.Errorf("tap name too long %s", name)
+	}
+	return name, nil
+}
+
+var ulaPrefix = net.IPNet{
+	IP:   net.ParseIP("fc00::"),
+	Mask: net.CIDRMask(7, 128),
+}
+
+func isULA(ip net.IP) bool {
+	return ulaPrefix.Contains(ip)
+}
