@@ -18,15 +18,17 @@ import (
 	"github.com/shirou/gopsutil/disk"
 	"github.com/threefoldtech/zos/pkg"
 	"github.com/threefoldtech/zos/pkg/app"
+	"github.com/threefoldtech/zos/pkg/capacity"
 	"github.com/threefoldtech/zos/pkg/storage/filesystem"
 )
 
 const (
 	// CacheTarget is the path where the cache disk is mounted
 	CacheTarget = "/var/cache"
-	cacheLabel  = "zos-cache"
-	gib         = 1024 * 1024 * 1024
-	cacheSize   = 100 * gib
+	// cacheLabel is the name of the cache
+	cacheLabel = "zos-cache"
+	gib        = 1024 * 1024 * 1024
+	cacheSize  = 100 * gib
 )
 
 var (
@@ -37,17 +39,20 @@ var (
 	}
 )
 
-type storageModule struct {
+// Module implements functionality for pkg.StorageModule
+type Module struct {
 	pools         []filesystem.Pool
 	brokenPools   []pkg.BrokenPool
 	devices       filesystem.DeviceManager
 	brokenDevices []pkg.BrokenDevice
+	totalSSD      uint64
+	totalHDD      uint64
 
 	mu sync.RWMutex
 }
 
 // New create a new storage module service
-func New() (pkg.StorageModule, error) {
+func New() (*Module, error) {
 	ctx, cancel := context.WithTimeout(context.Background(), time.Second*30)
 	defer cancel()
 
@@ -58,7 +63,7 @@ func New() (pkg.StorageModule, error) {
 		return nil, err
 	}
 
-	s := &storageModule{
+	s := &Module{
 		pools:         []filesystem.Pool{},
 		brokenPools:   []pkg.BrokenPool{},
 		devices:       m,
@@ -80,61 +85,35 @@ func New() (pkg.StorageModule, error) {
 }
 
 // Total gives the total amount of storage available for a device type
-func (s *storageModule) Total(kind pkg.DeviceType) (uint64, error) {
+func (s *Module) Total(kind pkg.DeviceType) (uint64, error) {
 	s.mu.RLock()
 	defer s.mu.RUnlock()
-	var total uint64
 
-	for _, pool := range s.pools {
-		// ignore pools which don't have the right device type
-		if pool.Type() != kind {
-			continue
-		}
-
-		unmountAfter := false
-		if _, mounted := pool.Mounted(); !mounted {
-			_, err := pool.MountWithoutScan()
-			if err != nil {
-				log.Error().Err(err).Msgf("Failed to mount pool %s", pool.Name())
-				return 0, err
-			}
-			unmountAfter = true
-		}
-
-		usage, err := pool.Usage()
-		if err != nil {
-			log.Error().Msgf("Failed to get current volume usage: %v", err)
-			return 0, err
-		}
-
-		total += usage.Size
-
-		if unmountAfter {
-			err := pool.UnMount()
-			if err != nil {
-				log.Error().Err(err).Msgf("Failed to unmount pool %s", pool.Name())
-				return 0, err
-			}
-		}
+	switch kind {
+	case pkg.SSDDevice:
+		return s.totalSSD, nil
+	case pkg.HDDDevice:
+		return s.totalHDD, nil
+	default:
+		return 0, fmt.Errorf("kind %+v unknown", kind)
 	}
-	return total, nil
 }
 
 // BrokenPools lists the broken storage pools that have been detected
-func (s *storageModule) BrokenPools() []pkg.BrokenPool {
+func (s *Module) BrokenPools() []pkg.BrokenPool {
 	s.mu.RLock()
 	defer s.mu.RUnlock()
 	return s.brokenPools
 }
 
 // BrokenDevices lists the broken devices that have been detected
-func (s *storageModule) BrokenDevices() []pkg.BrokenDevice {
+func (s *Module) BrokenDevices() []pkg.BrokenDevice {
 	s.mu.RLock()
 	defer s.mu.RUnlock()
 	return s.brokenDevices
 }
 
-func (s *storageModule) Dump() {
+func (s *Module) dump() {
 	log.Debug().Int("volumes", len(s.pools)).Msg("dumping volumes")
 
 	for _, pool := range s.pools {
@@ -157,7 +136,7 @@ What Initialize will do is the following:
  - Scan free devices, apply the policy.
  - If new pools were created, the pool is going to be mounted automatically
 **/
-func (s *storageModule) initialize(policy pkg.StoragePolicy) error {
+func (s *Module) initialize(policy pkg.StoragePolicy) error {
 	// lock for the entire initialization method, so other code which relies
 	// on this observes this as an atomic operation
 	s.mu.Lock()
@@ -208,7 +187,7 @@ func (s *storageModule) initialize(policy pkg.StoragePolicy) error {
 	sort.Sort(filesystem.ByReadTime(freeDisks))
 
 	// dumping current s.volumes list
-	s.Dump()
+	s.dump()
 
 	log.Info().Msgf("Creating new volumes using policy: %s", policy.Raid)
 
@@ -285,10 +264,26 @@ func (s *storageModule) initialize(policy pkg.StoragePolicy) error {
 	}
 
 	// add expvar variables
+	s.totalSSD = 0
+	s.totalHDD = 0
 	for _, pool := range s.pools {
 		for _, device := range pool.Devices() {
 			device.ShutdownCount = expvar.NewInt(device.Path)
 		}
+
+		// calculate size for pool
+		usage, err := pool.Usage()
+		if err != nil {
+			log.Error().Msgf("Failed to get current volume usage: %v", err)
+			continue
+		}
+
+		switch pool.Type() {
+		case pkg.HDDDevice:
+			s.totalHDD += usage.Size
+		case pkg.SSDDevice:
+			s.totalSSD += usage.Size
+		}
 	}
 
 	if err := filesystem.Partprobe(ctx); err != nil {
@@ -300,6 +295,14 @@ func (s *storageModule) initialize(policy pkg.StoragePolicy) error {
 		return err
 	}
 
+	hyperVisor, err := capacity.NewResourceOracle(nil).GetHypervisor()
+	if err == nil {
+		// Disable disk shutdown when running in a VM
+		if len(hyperVisor) > 0 {
+			return nil
+		}
+	}
+
 	if err := s.shutdownUnusedPools(); err != nil {
 		log.Error().Err(err).Msg("Error shutting down unused pools")
 	}
@@ -309,7 +312,7 @@ func (s *storageModule) initialize(policy pkg.StoragePolicy) error {
 	return nil
 }
 
-func (s *storageModule) shutdownUnusedPools() error {
+func (s *Module) shutdownUnusedPools() error {
 	for _, pool := range s.pools {
 		if _, mounted := pool.Mounted(); mounted {
 			volumes, err := pool.Volumes()
@@ -336,23 +339,39 @@ func (s *storageModule) shutdownUnusedPools() error {
 }
 
 // CreateFilesystem with the given size in a storage pool.
-func (s *storageModule) CreateFilesystem(name string, size uint64, poolType pkg.DeviceType) (string, error) {
+func (s *Module) CreateFilesystem(name string, size uint64, poolType pkg.DeviceType) (pkg.Filesystem, error) {
 	log.Info().Msgf("Creating new volume with size %d", size)
 	if strings.HasPrefix(name, "zdb") {
-		return "", fmt.Errorf("invalid volume name. zdb prefix is reserved")
+		return pkg.Filesystem{}, fmt.Errorf("invalid volume name. zdb prefix is reserved")
 	}
 
-	fs, err := s.createSubvol(size, name, poolType)
+	fs, err := s.createSubvolWithQuota(size, name, poolType)
 	if err != nil {
-		return "", err
+		return pkg.Filesystem{}, err
 	}
-	return fs.Path(), nil
+
+	usage, err := fs.Usage()
+	if err != nil {
+		return pkg.Filesystem{}, err
+	}
+
+	return pkg.Filesystem{
+		ID:     fs.ID(),
+		FsType: fs.FsType(),
+		Name:   fs.Name(),
+		Path:   fs.Path(),
+		Usage: pkg.Usage{
+			Size: usage.Size,
+			Used: usage.Used,
+		},
+		DiskType: poolType,
+	}, nil
 }
 
 // ReleaseFilesystem with the given name, this will unmount and then delete
 // the filesystem. After this call, the caller must not perform any more actions
 // on this filesystem
-func (s *storageModule) ReleaseFilesystem(name string) error {
+func (s *Module) ReleaseFilesystem(name string) error {
 	log.Info().Msgf("Deleting volume %v", name)
 
 	for _, pool := range s.pools {
@@ -393,26 +412,158 @@ func (s *storageModule) ReleaseFilesystem(name string) error {
 	return nil
 }
 
+// ListFilesystems return all the filesystem managed by storeaged present on the nodes
+func (s *Module) ListFilesystems() ([]pkg.Filesystem, error) {
+	fss := make([]pkg.Filesystem, 0, 10)
+
+	for _, pool := range s.pools {
+		if _, mounted := pool.Mounted(); !mounted {
+			continue
+		}
+
+		volumes, err := pool.Volumes()
+		if err != nil {
+			return nil, err
+		}
+		for _, v := range volumes {
+			// Do not return "special" volumes here
+			// instead the GetCacheFS and GetVdiskFS to access them
+			if v.Name() == cacheLabel ||
+				v.Name() == vdiskVolumeName {
+				continue
+			}
+
+			usage, err := v.Usage()
+			if err != nil {
+				return nil, err
+			}
+
+			fss = append(fss, pkg.Filesystem{
+				ID:     v.ID(),
+				FsType: v.FsType(),
+				Name:   v.Name(),
+				Path:   v.Path(),
+				Usage: pkg.Usage{
+					Size: usage.Size,
+					Used: usage.Used,
+				},
+				DiskType: pool.Type(),
+			})
+		}
+	}
+
+	return fss, nil
+}
+
 // Path return the path of the mountpoint of the named filesystem
 // if no volume with name exists, an empty path and an error is returned
-func (s *storageModule) Path(name string) (string, error) {
-	for idx := range s.pools {
-		filesystems, err := s.pools[idx].Volumes()
-		if err != nil {
-			return "", err
+func (s *Module) Path(name string) (pkg.Filesystem, error) {
+	_, fs, err := s.path(name)
+	return fs, err
+}
+
+// Path return the path of the mountpoint of the named filesystem
+// if no volume with name exists, an empty path and an error is returned
+func (s *Module) path(name string) (filesystem.Pool, pkg.Filesystem, error) {
+	for _, pool := range s.pools {
+		if _, mounted := pool.Mounted(); !mounted {
+			continue
 		}
-		for jdx := range filesystems {
-			if filesystems[jdx].Name() == name {
-				return filesystems[jdx].Path(), nil
+		filesystems, err := pool.Volumes()
+		if err != nil {
+			return nil, pkg.Filesystem{}, err
+		}
+		for _, fs := range filesystems {
+			if fs.Name() == name {
+				usage, err := fs.Usage()
+				if err != nil {
+					return nil, pkg.Filesystem{}, err
+				}
+
+				return pool, pkg.Filesystem{
+					ID:     fs.ID(),
+					FsType: fs.FsType(),
+					Name:   fs.Name(),
+					Path:   fs.Path(),
+					Usage: pkg.Usage{
+						Size: usage.Size,
+						Used: usage.Used,
+					},
+					DiskType: pool.Type(),
+				}, nil
 			}
 		}
 	}
 
-	return "", errors.Wrapf(os.ErrNotExist, "subvolume '%s' not found", name)
+	return nil, pkg.Filesystem{}, errors.Wrapf(os.ErrNotExist, "subvolume '%s' not found", name)
+}
+
+// VDiskFindCandidate find a suitbale location for creating a vdisk of the given size
+func (s *Module) VDiskFindCandidate(size uint64) (path string, err error) {
+	candidates, err := s.findCandidates(size, pkg.SSDDevice)
+	if err != nil {
+		return path, err
+	}
+	// does anyone have a vdisk subvol
+	for _, candidate := range candidates {
+		volumes, err := candidate.Pool.Volumes()
+		if err != nil {
+			log.Error().Str("pool", candidate.Pool.Path()).Err(err).Msg("failed to list pool volumes")
+			continue
+		}
+		for _, volume := range volumes {
+			if volume.Name() != vdiskVolumeName {
+				continue
+			}
+
+			return volume.Path(), nil
+		}
+	}
+	// none has a vdiks subvolume, we need to
+	// create one.
+	candidate := candidates[0]
+	volume, err := candidate.Pool.AddVolume(vdiskVolumeName)
+	if err != nil {
+		return path, errors.Wrap(err, "failed to create vdisk pool")
+	}
+
+	return volume.Path(), nil
+}
+
+// VDiskPools return a list of all vdisk pools
+func (s *Module) VDiskPools() ([]string, error) {
+	var paths []string
+	for _, pool := range s.pools {
+		if pool.Type() != pkg.SSDDevice {
+			continue
+		}
+
+		if _, mounted := pool.Mounted(); !mounted {
+			continue
+		}
+
+		volumes, err := pool.Volumes()
+		if err != nil {
+			return nil, errors.Wrapf(err, "failed to list pool '%s' volumes", pool.Path())
+		}
+
+		for _, volume := range volumes {
+			if volume.Name() == vdiskVolumeName {
+				paths = append(paths, volume.Path())
+			}
+		}
+	}
+
+	return paths, nil
+}
+
+// GetCacheFS return the special filesystem used by 0-OS to store internal state and flist cache
+func (s *Module) GetCacheFS() (pkg.Filesystem, error) {
+	return s.Path(cacheLabel)
 }
 
 // ensureCache creates a "cache" subvolume and mounts it in /var
-func (s *storageModule) ensureCache() error {
+func (s *Module) ensureCache() error {
 	log.Info().Msgf("Setting up cache")
 
 	log.Debug().Msgf("Checking pools for existing cache")
@@ -420,15 +571,18 @@ func (s *storageModule) ensureCache() error {
 	var cacheFs filesystem.Volume
 
 	// check if cache volume available
-	for idx := range s.pools {
-		filesystems, err := s.pools[idx].Volumes()
+	for _, pool := range s.pools {
+		if _, mounted := pool.Mounted(); !mounted {
+			continue
+		}
+		filesystems, err := pool.Volumes()
 		if err != nil {
 			return err
 		}
-		for jdx := range filesystems {
-			if filesystems[jdx].Name() == cacheLabel {
-				log.Debug().Msgf("Found existing cache at %v", filesystems[jdx].Path())
-				cacheFs = filesystems[jdx]
+		for _, fs := range filesystems {
+			if fs.Name() == cacheLabel {
+				log.Debug().Msgf("Found existing cache at %v", fs.Path())
+				cacheFs = fs
 				break
 			}
 		}
@@ -441,7 +595,7 @@ func (s *storageModule) ensureCache() error {
 		log.Debug().Msgf("No cache found, try to create new cache")
 
 		log.Debug().Msgf("Trying to create new cache on SSD")
-		fs, err := s.createSubvol(cacheSize, cacheLabel, pkg.SSDDevice)
+		fs, err := s.createSubvolWithQuota(cacheSize, cacheLabel, pkg.SSDDevice)
 
 		if err != nil {
 			log.Warn().Err(err).Msg("failed to create new cache on SSD")
@@ -452,7 +606,7 @@ func (s *storageModule) ensureCache() error {
 
 	if cacheFs == nil {
 		log.Debug().Msgf("Trying to create new cache on HDD")
-		fs, err := s.createSubvol(cacheSize, cacheLabel, pkg.HDDDevice)
+		fs, err := s.createSubvolWithQuota(cacheSize, cacheLabel, pkg.HDDDevice)
 
 		if err != nil {
 			log.Warn().Err(err).Msg("failed to create new cache on HDD")
@@ -487,10 +641,27 @@ func (s *storageModule) ensureCache() error {
 	return nil
 }
 
-// createSubvol creates a subvolume with the given name and limits it to the given size
-// if the requested disk type does not have a storage pool available, an error is
-// returned
-func (s *storageModule) createSubvol(size uint64, name string, poolType pkg.DeviceType) (filesystem.Volume, error) {
+// createSubvolWithQuota creates a subvolume with the given name and limits it to the given size
+// if the requested disk type does not have a storage pool with enough free size available, an error is returned
+// this methods does set a quota limit equal to size on the created volume
+func (s *Module) createSubvolWithQuota(size uint64, name string, poolType pkg.DeviceType) (filesystem.Volume, error) {
+	volume, err := s.createSubvol(size, name, poolType)
+	if err != nil {
+		return nil, err
+	}
+
+	if err = volume.Limit(size); err != nil {
+		log.Error().Err(err).Str("volume", volume.Path()).Msg("failed to set volume size limit")
+		return nil, err
+	}
+
+	return volume, nil
+}
+
+// createSubvol creates a subvolume with the given name
+// if the requested disk type does not have a storage pool with enough free size available, an error is returned
+// this method does not set any quota on the subvolume, for this uses createSubvolWithQuota
+func (s *Module) createSubvol(size uint64, name string, poolType pkg.DeviceType) (filesystem.Volume, error) {
 	var err error
 
 	if poolType != pkg.HDDDevice && poolType != pkg.SSDDevice {
@@ -498,34 +669,12 @@ func (s *storageModule) createSubvol(size uint64, name string, poolType pkg.Devi
 	}
 
 	// Look for candidates in mounted pools first
-	candidates, err := s.checkForCandidates(size, poolType, true)
+	candidates, err := s.findCandidates(size, poolType)
 	if err != nil {
 		log.Error().Err(err).Msgf("failed to search candidates on mounted pools")
 		return nil, err
 	}
 
-	log.Debug().Msgf("Found %d candidates in mounted pools", len(candidates))
-
-	// If no candidates or found in mounted pools, we check the unmounted pools and get the first one that fits
-	if len(candidates) == 0 {
-		log.Debug().Msg("Checking unmounted pools")
-		candidates, err = s.checkForCandidates(size, poolType, false)
-		if err != nil {
-			log.Error().Err(err).Msgf("failed to search candidates on mounted pools")
-			return nil, err
-		}
-		log.Debug().Msgf("Found %d candidates in unmounted pools", len(candidates))
-	}
-
-	if len(candidates) == 0 {
-		return nil, pkg.ErrNotEnoughSpace{DeviceType: poolType}
-	}
-
-	sort.Slice(candidates, func(i, j int) bool {
-		// reverse sorting so most available is at beginning
-		return candidates[i].Available > candidates[j].Available
-	})
-
 	var volume filesystem.Volume
 	for _, candidate := range candidates {
 		volume, err = candidate.Pool.AddVolume(name)
@@ -533,11 +682,6 @@ func (s *storageModule) createSubvol(size uint64, name string, poolType pkg.Devi
 			log.Error().Err(err).Str("pool", candidate.Pool.Name()).Msg("failed to create new filesystem")
 			continue
 		}
-		if err = volume.Limit(size); err != nil {
-			candidate.Pool.RemoveVolume(volume.Name()) // try to recover
-			log.Error().Err(err).Str("volume", volume.Path()).Msg("failed to set volume size limit")
-			continue
-		}
 
 		return volume, nil
 	}
@@ -550,7 +694,35 @@ type candidate struct {
 	Available uint64
 }
 
-func (s *storageModule) checkForCandidates(size uint64, poolType pkg.DeviceType, mounted bool) ([]candidate, error) {
+func (s *Module) findCandidates(size uint64, poolType pkg.DeviceType) ([]candidate, error) {
+
+	// Look for candidates in mounted pools first
+	candidates, err := s.checkForCandidates(size, poolType, true)
+	if err != nil {
+		return nil, errors.Wrap(err, "failed to search candidate on mounted pools")
+	}
+
+	log.Debug().Msgf("found %d candidates in mounted pools", len(candidates))
+
+	// If no candidates or found in mounted pools, we check the unmounted pools and get the first one that fits
+	if len(candidates) == 0 {
+		log.Debug().Msg("Checking unmounted pools")
+		candidates, err = s.checkForCandidates(size, poolType, false)
+		if err != nil {
+			return nil, errors.Wrap(err, "failed to search candidates on unmounted pools")
+		}
+
+		log.Debug().Msgf("found %d candidates in unmounted pools", len(candidates))
+	}
+
+	if len(candidates) == 0 {
+		return nil, pkg.ErrNotEnoughSpace{DeviceType: poolType}
+	}
+
+	return candidates, nil
+}
+
+func (s *Module) checkForCandidates(size uint64, poolType pkg.DeviceType, mounted bool) ([]candidate, error) {
 	var candidates []candidate
 	for _, pool := range s.pools {
 		_, poolIsMounted := pool.Mounted()
@@ -621,10 +793,17 @@ func (s *storageModule) checkForCandidates(size uint64, poolType pkg.DeviceType,
 			return candidates, nil
 		}
 	}
+
+	sort.Slice(candidates, func(i, j int) bool {
+		// reverse sorting so most available is at beginning
+		return candidates[i].Available > candidates[j].Available
+	})
+
 	return candidates, nil
 }
 
-func (s *storageModule) Monitor(ctx context.Context) <-chan pkg.PoolsStats {
+// Monitor implements monitor method
+func (s *Module) Monitor(ctx context.Context) <-chan pkg.PoolsStats {
 	ch := make(chan pkg.PoolsStats)
 	values := make(pkg.PoolsStats)
 	go func() {
@@ -638,6 +817,9 @@ func (s *storageModule) Monitor(ctx context.Context) <-chan pkg.PoolsStats {
 			}
 
 			for _, pool := range s.pools {
+				if _, mounted := pool.Mounted(); !mounted {
+					continue
+				}
 				devices, err := s.devices.ByLabel(ctx, pool.Name())
 				if err != nil {
 					log.Error().Err(err).Str("pool", pool.Name()).Msg("failed to get devices for pool")
@@ -680,7 +862,7 @@ func (s *storageModule) Monitor(ctx context.Context) <-chan pkg.PoolsStats {
 	return ch
 }
 
-func (s *storageModule) periodicallyCheckDiskShutdown() {
+func (s *Module) periodicallyCheckDiskShutdown() {
 	ticker := time.NewTicker(5 * time.Minute)
 
 	go func() {
@@ -694,7 +876,7 @@ func (s *storageModule) periodicallyCheckDiskShutdown() {
 
 // shutdownDisks will check the disks power status.
 // If a disk is on and it is not mounted then it is not supposed to be on, turn it off
-func (s *storageModule) shutdownDisks() {
+func (s *Module) shutdownDisks() {
 	for _, pool := range s.pools {
 		for _, device := range pool.Devices() {
 			log.Debug().Msgf("checking device: %s", device.Path)
